{$N+}

UNIT GritCalc;

INTERFACE

uses Dos,Crt;

const version_id='GRIT for Calcomp 565, version 0.0 / October, 2004';

(* e.h. dooijes / computer museum, university of amsterdam *)

      closed   = true;
      nonclosed = false;

type realarray = array[1..200] of real;

procedure SetupPlotter;
procedure SetWindow(xmin,ymin,xmax,ymax: real);
procedure DrawLine(x1,y1,x2,y2: real);
procedure Polyline(x,y: realarray; n: integer; close_polygon: boolean);
procedure DonePlotter;


IMPLEMENTATION


const maxsteps = 3048; {12 inch width, step size 0.1 mm}
      pen_down = true;
      pen_up = false;

var AX,BX,AY,BY: real;
    lastx,lasty: integer;
    f: text;


procedure move(xx,yy,zz: integer);  {1 step in x y and/or z direction}
var
   actie: byte;
   cr,cl,du,dd,pu,pd: integer;
        {carriage left/right, drum up/down, pen up/down}
begin
   cr:=0; cl:=0; du:=0; dd:=0; pu:=0; pd:=0;
   if xx>0 then cr:=1; if xx<0 then cl:=1;
   if yy>0 then dd:=1; if yy<0 then du:=1;
   if zz>0 then pu:=1; if zz<0 then pd:=1;
   actie:=cr+2*cl+4*du+8*dd+16*pu+32*pd;
   port[$213]:=$80;      {configure ports as outputs}
   port[$210]:=actie;    {set port a , plotter moving}
   delay(1);
   port[$210]:=$00;      {plotter in rest}
   delay(3);  {total delay >=4 msec  (250 steps/sec)}
   if zz<>0 then delay(100);   {at pen up or down >=100 msec delay}
   lastx:=lastx+cr-cl; lasty:=lasty+dd-du;
end;

procedure bresenham(x2,y2: integer; pendown: boolean);
{x2,y2: end point coordinates relative to starting point}
{adapted from Rogers, Procedural Elements}
var i,dx,dy,e,s1,s2,x,y,temp: integer;
    swap: boolean;
    xsav,ysav: integer;

function signum(x: integer): integer;
begin
   if x<0 then signum:=-1
          else begin if x>0 then signum:=1 else signum:=0 end
end;

begin
   if pendown then move(0,0,-1) else move(0,0,1);
   x:=0;
   y:=0;
   dx:=abs(x2);
   dy:=abs(y2);
   s1:=signum(x2);
   s2:=signum(y2);
   if dy>dx then
      begin
         temp:=dx;
         dx:=dy;
         dy:=temp;
         swap:=true
      end
   else swap:=false;
   e:=2*dy-dx;
   for i:=1 to dx do
     begin
         xsav:=x; ysav:=y;
         while e>=0 do
            begin
                if swap then x:=x+s1
                        else y:=y+s2;
                e:=e-2*dx
            end;
         if swap then y:=y+s2
                 else x:=x+s1;
         e:=e+2*dy;
         move(x-xsav,y-ysav,0)
     end
end;



procedure SetupPlotter;
var c: char;
begin
  writeln('put pen in left starting position, then hit a key');
  repeat until keypressed; c:=readkey;
  lastx:=0; lasty:=0
end;



procedure SetWindow;  {isotrope}
begin
   AX:=maxsteps/(xmax-xmin); BX:=-xmin/(xmax-xmin);
   AY:=AX;                   BY:=BX
end;

procedure DrawLine;
var ix1,iy1,ix2,iy2:integer;
begin
   ix1:=round(AX*x1+BX); iy1:=round(AY*y1+BY);
   ix2:=round(AX*x2+BX); iy2:=round(AY*y2+BY);
   bresenham(ix1-lastx,iy1-lasty,pen_up); {lastx,lasty updated by move}
   bresenham(ix2-lastx,iy2-lasty,pen_down);
end;

procedure Polyline;
var i:integer;
    ix,iy,ix1,iy1: integer;
begin
  ix1:=round(AX*x[1]+BX); iy1:=round(AY*y[1]+BY);
  bresenham(ix1-lastx,iy1-lasty,pen_up);
  for i:=2 to n do
    begin
       ix:=round(AX*x[i]+BX); iy:=round(AY*y[i]+BY);
       bresenham(ix-lastx,iy-lasty,pen_down);
    end;
  if close_polygon then
    begin
       bresenham(ix1-lastx,iy1-lasty,pen_down);
    end
end;

procedure DonePlotter;
begin
   bresenham(-lastx,-lasty,0,0,pen_up)
end;

end.
